<div class="addons-wrap d-flex flex-column overflow-y-auto">
    <div class="status d-flex flex-justify-between p-2">
        <div class="title p-1">
            <i class="fa fa-book"></i> {{ site.title }}
        </div>
        <div class="branch p-1">
            <i class="fa fa-caret-down"></i>
        </div>
    </div>
    <div class="addons d-flex flex-column height-full p-2 d-none">
        <dl>
            <dt>{{ __.github | default: "GitHub" }}</dt>
            <dd>
                <a href="{{ docs.repository_url }}" title="{{ __.stars | default: 'Stars' }}: {{ rest.stargazers_count }}">
                    <i class="fa fa-github"></i>
                    {{ __.homepage | default: "Homepage" }}
                </a>
            </dd>
            <dd>
                <a href="{{ docs.issues_url }}" title="{{ __.open_issues | default: 'Open issues' }}: {{ rest.open_issues }}">
                    <i class="fa fa-question-circle-o"></i>
                    {{ __.issues | default: "Issues" }}
                </a>
            </dd>
            <dd>
            <a href="{{ site.additional.metacpan_url }}" title="{{ __.size | default: 'Size' }}: {{ rest.size }} Kb">
                    <i class="fa fa-download"></i>
                    {{ __.download | default: "MetaCPAN" }}
                </a>
            </dd>
        </dl>
        <hr>
        <div class="license f6 pb-2">
            This
            <a href="{{ docs.repository_url }}/" title="Software">Software</a>
            is under the terms of
            <a href="{{ site.additional.license.url }}">{{ site.additional.license.name | default: "The Unlicense" }}</a>.
        </div>
    </div>
</div>
